///*****************************************************************************
//*
//* Copyright (C) 2012 Ittiam Systems Pvt Ltd, Bangalore
//*
//* Licensed under the Apache License, Version 2.0 (the "License");
//* you may not use this file except in compliance with the License.
//* You may obtain a copy of the License at:
//*
//* http://www.apache.org/licenses/LICENSE-2.0
//*
//* Unless required by applicable law or agreed to in writing, software
//* distributed under the License is distributed on an "AS IS" BASIS,
//* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* See the License for the specific language governing permissions and
//* limitations under the License.
//*
//*****************************************************************************/
///**
//*******************************************************************************
//* @file
//*  ihevc_intra_pred_luma_mode_18_34_neon.s
//*
//* @brief
//*  contains function definitions for intra prediction dc filtering.
//* functions are coded using neon  intrinsics and can be compiled using

//* rvct
//*
//* @author
//*  yogeswaran rs
//*
//* @par list of functions:
//*
//*
//* @remarks
//*  none
//*
//*******************************************************************************
//*/
///**
//*******************************************************************************
//*
//* @brief
//*    luma intraprediction filter for dc input
//*
//* @par description:
//*
//* @param[in] pu1_ref
//*  uword8 pointer to the source
//*
//* @param[out] pu1_dst
//*  uword8 pointer to the destination
//*
//* @param[in] src_strd
//*  integer source stride
//*
//* @param[in] dst_strd
//*  integer destination stride
//*
//* @param[in] pi1_coeff
//*  word8 pointer to the planar coefficients
//*
//* @param[in] nt
//*  size of tranform block
//*
//* @param[in] mode
//*  type of filtering
//*
//* @returns
//*
//* @remarks
//*  none
//*
//*******************************************************************************
//*/

//void ihevc_intra_pred_chroma_mode_18_34(uword8 *pu1_ref,
//                                      word32 src_strd,
//                                      uword8 *pu1_dst,
//                                      word32 dst_strd,
//                                      word32 nt,
//                                      word32 mode)
//
//**************variables vs registers*****************************************
//x0 => *pu1_ref
//x1 => src_strd
//x2 => *pu1_dst
//x3 => dst_strd

//stack contents from #40
//    nt
//    mode
//    pi1_coeff

.text
.align 4
.include "ihevc_neon_macros.s"



.globl ihevc_intra_pred_chroma_mode_18_34_av8

.type ihevc_intra_pred_chroma_mode_18_34_av8, %function

ihevc_intra_pred_chroma_mode_18_34_av8:

    // stmfd sp!, {x4-x12, x14}    //stack stores the values of the arguments

    stp         x19, x20,[sp,#-16]!


    cmp         x4,#4
    beq         mode2_4

    mov         x12,x4
    mov         x11,x4
    add         x0,x0,x4,lsl #2

    cmp         x5,#0x22
    mov         x10,x2

    add         x0,x0,#4

    sub         x20,x0,#4
    csel        x0, x20, x0,ne
    mov         x20,#2
    csel        x6, x20, x6,eq
    mov         x20,#-2
    csel        x6, x20, x6,ne
    mov         x8,x0


kernel:


    ld1         {v0.8b, v1.8b},[x8],x6
    st1         {v0.8b, v1.8b},[x10],x3
    ld1         {v2.8b, v3.8b},[x8],x6
    st1         {v2.8b, v3.8b},[x10],x3
    ld1         {v4.8b, v5.8b},[x8],x6
    st1         {v4.8b, v5.8b},[x10],x3
    ld1         {v6.8b, v7.8b},[x8],x6
    st1         {v6.8b, v7.8b},[x10],x3
    ld1         {v16.8b, v17.8b},[x8],x6
    st1         {v16.8b, v17.8b},[x10],x3
    ld1         {v18.8b, v19.8b},[x8],x6
    st1         {v18.8b, v19.8b},[x10],x3
    ld1         {v20.8b, v21.8b},[x8],x6
    st1         {v20.8b, v21.8b},[x10],x3
    ld1         {v22.8b, v23.8b},[x8],x6
    st1         {v22.8b, v23.8b},[x10],x3

    subs        x12,x12,#8
    bne         kernel

    cmp         x11,#16
    add         x8,x0,#16
    add         x10,x2,#16
    sub         x11, x11,#16
    mov         x12,#16
    beq         kernel
    b           end_func

mode2_4:

    add         x0,x0,#20
    cmp         x5,#0x22
    sub         x20,x0,#4
    csel        x0, x20, x0,ne

    mov         x20,#2
    csel        x8, x20, x8,eq
    mov         x20,#-2
    csel        x8, x20, x8,ne

    ld1         {v0.8b},[x0],x8
    st1         {v0.2s},[x2],x3

    ld1         {v0.8b},[x0],x8
    st1         {v0.2s},[x2],x3

    ld1         {v0.8b},[x0],x8
    st1         {v0.2s},[x2],x3

    ld1         {v0.8b},[x0],x8
    st1         {v0.2s},[x2],x3

end_func:
    // ldmfd sp!,{x4-x12,x15}                  //reload the registers from sp
    ldp         x19, x20,[sp],#16

    ret






